<?php

/**
 * Класс для упрощения работы со средствами SAX встроеных в PHP.
 *
 * Создает дерево в виде массива, принимает путь к файлу
 * @author IvanSCM <ivanscm@fromru.com>
 */
class SAXRead {

    /**
     *
     * @var string $filename Имя файла для работы внутри класса
     */
    private $filename;

    /**
     *
     * @var resource $xml Дескриптор XML парсера
     */
    private $xml;

    /**
     *
     * @var integer $level Текущий уровень вложености при обходе
     */
    private $level;

    /**
     *
     * @var array $tree Дерево документа
     */
    private $tree;

    private $stack;

    /**
     * Конструктор
     *
     * Устанавливает имя файла для дальнейшей работы класса и регистрирует методы для обработчиков SAX
     * @param string $filename
     */
    function __construct($filename) {
        $this->level = 0;
        $this->filename = $filename;
        $this->xml = xml_parser_create('UTF-8');
        xml_set_element_handler($this->xml, 'self::StartTAG', 'self::EndTAG');
        xml_set_character_data_handler($this->xml, 'self::CharFind');
        //устанавливаем нижний регистр для тегов документа
        xml_parser_set_option($this->xml, XML_OPTION_CASE_FOLDING, false);
    }

    /**
     * Обработчик начального тега
     * @param resouce $xml
     * @param mixed $tag
     * @param mixed $attr
     */
    private function  StartTAG ($xml, $tag, $attr){
        if ($this->level == 0) {
            $this->tree['name'] = $tag;
            foreach ($attr as $key => $val) {
               $this->tree['attributes'][$key] = $val;  
            }
        }
        if ($this->level > 0) {
            foreach ($attr as $key => $val) {
               $attributes[$key] = $val;
            }
            $this->stack[] = array('children' => array('name'=> $tag, 'attributes'=>$attributes));
        }
        $this->level++;
    }

    /**
     * Обработчик конечного тега
     * @param resource $xml
     * @param mixed $tag
     */
    private function EndTag($xml, $tag) {
        if ($this->level == 1) {
            $this->tree['children'][] = $this->stack;
            $this->stack = array();
        }
        $this->level--;
    }

    /**
     * Обработчик содержимого
     * @param resourse $xml
     * @param mixed $data
     */
    private static function CharFind($xml, $data){
        
    }
    
    /**
     * Функция ReadFile
     * 
     * Создает структуру из файла
     * @return Array
     */
    function ReadFile(){
        xml_parse($this->xml, file_get_contents($this->filename));
        return $this->tree;
    }

}
?>
